#ifndef PRINT_H
#define PRINT_H

/*
 * abcprintd internal data structures
 */

#include "abcprintd.h"
#include "hostfile.h"

/* Ordinary input state machine */
enum input_state {
    is_normal,                  /* Normal operation */
    is_ff,                      /* 0xFF received */
    is_file,                    /* File operation in progress */
    is_console                  /* Output to console */
};

/* File operations state machine */
enum fileop_state {
    st_op,                      /* Receiving command */
    st_open,
    st_read,
    st_print,                   /* Before data */
    st_print2,                  /* After data */
    st_seek,
    st_rename,
    st_delete,
    st_pread,
    st_pwrite,                  /* Before data */
    st_pwrite2,                 /* After data */
    st_blksize
};

struct fileop_file {
    struct host_file *hf;
    bool binary;
};

typedef union argbuf {
    uint8_t b[32];
    char c[32];
    uint64_t q;
} argbuf;

struct abcprint {
    /* Function to send data */
    struct send_data {
	send_func func;
	void *pvt;
    } sd;

    /* Temporary file for actual printing */
    struct host_file *prfile;

    /* Input state machine */
    enum input_state istate;

    /* File opeerations state machine */
    enum fileop_state fstate;

    /* System block size */
    unsigned int blksize;

    /* Current ix value, and corresponding fileop_file, if any */
    uint16_t ix;
    struct fileop_file *ff;

    /* Pending I/O data */
    unsigned int datalen;	/* Data expected for the main data buffer */
    unsigned int byte_count;	/* Data still required */
    unsigned char *bytep;	/* Pointer to next byte to be received */


    /* Data buffers */
    unsigned char cmd[4];	/* Buffer for command */
    argbuf argbuf;		/* Buffer for argument(s) */
    unsigned char data[65536+2]; /* Data buffer (maximum possible size) */

    /* Filemap; massive waste of space -- clean up? */
    struct fileop_file filemap[65536];
};

extern void fileop_reset(struct abcprint *me);
extern bool file_op(struct abcprint *me, unsigned char c);

#endif /* PRINT_H */
